@@ -177,6 +177,7 @@ void symbol::set_domain(unsigned d)
                         iflags.set(info_flags::positive, false);
                         iflags.set(info_flags::nonnegative, false);
                         iflags.set(info_flags::integer, false);
+                        iflags.set(info_flags::even, false);
                         break;
                 case domain::real:
                         iflags.set(info_flags::real, true);
@@ -195,9 +196,30 @@ void symbol::set_domain(unsigned d)
                                 iflags.set(info_flags::integer, false);
                         }
                         break;
+                        if(iflags.get(info_flags::even)) {
+                                iflags.set(info_flags::even, true);
+                        }
+                        else {
+                                iflags.set(info_flags::even, false);
+                        }
+                        break;
                 case domain::positive:
                         iflags.set(info_flags::real, true);
                         iflags.set(info_flags::positive, true);
+                        iflags.set(info_flags::nonnegative, true);
+                        if(iflags.get(info_flags::integer)) {
+                                iflags.set(info_flags::integer, true);
+                        }
+                        else {
+                                iflags.set(info_flags::integer, false);
+                        }
+                        break;
+                        if(iflags.get(info_flags::even)) {
+                                iflags.set(info_flags::even, true);
+                        }
+                        else {
+                                iflags.set(info_flags::even, false);
+                        }
                         break;
                 case domain::integer:
                         if(iflags.get(info_flags::positive)) {
@@ -206,8 +228,23 @@ void symbol::set_domain(unsigned d)
                         else {
                                 iflags.set(info_flags::positive, false);
                         }
-                        iflags.set(info_flags::real, true);
                         iflags.set(info_flags::integer, true);
+                        if(iflags.get(info_flags::even)) {
+                                iflags.set(info_flags::even, true);
+                        }
+                        else {
+                                iflags.set(info_flags::even, false);
+                        }
+                        break;
+                case domain::even:
+                        if(iflags.get(info_flags::positive)) {
+                                iflags.set(info_flags::positive, true);
+                        }
+                        else {
+                                iflags.set(info_flags::positive, false);
+                        }
+                        iflags.set(info_flags::integer, true);
+                        iflags.set(info_flags::even, true);
                         break;
         }
 }
@@ -224,6 +261,9 @@ bool symbol::info(unsigned inf) const
 		case info_flags::rational_function: 
 		case info_flags::expanded:
 			return true;
+                case info_flags::nonzero:
+                        return iflags.get(info_flags::positive)
+                            or iflags.get(info_flags::negative);
                 case info_flags::infinity:
                         return false;
                 default:
